/*
 * Library for drawing numeric data in diagram
 */

#ifndef A_OSCILL_H_
#define A_OSCILL_H_

/* screen width in oscill (in pixel) */
#define SCREEN_WIDTH  401
/* screen width in oscill (in pixel) */
#define SCREEN_HEIGHT 321

/* cell size in screnn of oscill (in pixel) */
#define CELL_PIXELS_WIDTH  40
#define CELL_PIXELS_HEIGHT 40

/* distance between cut in cell equal CELL_PIXELS_WIDTH / 5 */
#define NUM_PIXEL_DIV 8

#define NUM_COLOR 4

/* size buffer for interaction with oscill */
#define A_OSCILL_BUFF_SIZE 256

enum {
	A_OSCILL_OK  = 0,
	A_OSCILL_ERR = -1,
};

/* oscill screen colors */
enum {
	green,
	red,
	yellow,
	blue,
	black,
	white,
};

struct a_oscill;
struct circle_buff;

typedef void (*a_oscill_draw_func_t)(SDL_Surface *screen,
				struct circle_buff *c_b, int ccolor, int scolor);

typedef void (*a_oscill_deinit_func_t)(struct a_oscill **oscill);


struct circle_buff {
	short *buff;       /* buffer for storage data */

	int head, tail;    /* head - number write element, tail - number read element */

	int buff_size;     /* size of buffer */
};


struct a_oscill {
	SDL_Surface *screen;			  /* screen for drawing plot */

	struct circle_buff *c_buff;       /* data for curve in screen */

	double volt_div;                  /* division voltage */

	double time_div;                  /* division time */

	double i_time_div;                /* division input time */

	double solve_time;                /* time for approximation */

	double prev_sample;               /* previous sample */

	long int last_update_screen;      /* time last update screen */

	short t_screen_update;            /* period update screen (milliseconds)*/

	int ndata;                        /* number data for plot */

	int scolor;                       /* color of screen */

	int ccolor;                       /* color of curve */

	int flag;                         /*  */

	a_oscill_draw_func_t a_oscill_update_screen; /* update image in screen */

	a_oscill_deinit_func_t a_oscill_deinit;   /* deinit struct a_oscill */
};

/*!
 * a_oscill_new - create struct a_oscill.
 * /screencolor: color of screen in struct a_oscill
 * curvecolor : color of curve in screen
 *
 * return: NULL if error
 *         pointer to struct a_oscill else
 */
struct a_oscill *a_oscill_new(int screencolor, int curvecolor);

/**
 * a_oscill_update_screen - draw new image in field screen in struct a_oscill.
 * @screen: pointer to graphical surface structure
 * @c_b   : pointer to data for curve in screen (circular buffer)
 * @ccolor: color of curve in screen
 * @scolor: color of screen
 */
void a_oscill_update_screen(SDL_Surface *screen, struct circle_buff *c_b, int ccolor, int scolor);

/**
 * a_oscill_set_time_div - change field time_div in struct a_oscill
 * @time_div: horizontal scale division in screen in struct a_oscill
 */
void a_oscill_set_time_div(struct a_oscill *oscill, double time_div);

/**
 * a_oscill_get_time_div - output value the field time_div from struct oscill on stdout
 * @data: user data
 */
void a_oscill_get_time_div(struct a_oscill *oscill, void *data);

/**
 * a_oscill_set_i_time_div - change field i_time_div (input time) in struct a_oscill
 * @i_time_div: time division of input data
 */
void a_oscill_set_i_time_div(struct a_oscill *oscill, double i_time_div);

/**
 * a_oscill_get_i_time_div - output value the field i_time_div from struct oscill on stdout
 * @data: user data
 */
void a_oscill_get_i_time_div(struct a_oscill *oscill, void *data);

/**
 * a_oscill_set_volt_di - change field volt_div in struct a_oscill
 * @volt_div: vertical scale division in screen in struct a_oscill
 */
void a_oscill_set_volt_div(struct a_oscill *oscill, double volt_div);

/**
 * a_oscill_get_volt_div - output value the field volt_div from struct oscill on stdout
 * @data: user data
 */
void a_oscill_get_volt_div(struct a_oscill *oscill, void *data);

/**
 * a_oscill_set_scolor - change field scolor (screen color) in struct a_oscill
 * @scolor = red, blue, yellow or green
 * for example: a_oscill_set_scolor(oscill, red)
 */
void a_oscill_set_scolor(struct a_oscill *oscill, int scolor);

/**
 * a_oscill_get_scolor - output value the field scolor from struct oscill on stdout
 * @data: user data
 */
void a_oscill_get_scolor(struct a_oscill *oscill, void *data);

/**
 * a_oscill_set_ccolor - change field ccolor (curve color) in struct a_oscill
 * @ccolor: curve color = red, blue, yellow or green
 * for example: a_oscill_set_ccolor(oscill, red)
 */
void a_oscill_set_ccolor(struct a_oscill *oscill, int ccolor);

/**
 * a_oscill_get_ccolor - output value the field ccolor from struct oscill on stdout
 * @data: user data
 */
void a_oscill_get_ccolor(struct a_oscill *oscill, void *data);

/**
 * a_oscill_set_t_update_screen - change field t_screen_update in struct a_oscill
 * @t_screen_update: period of update screen in struct a_oscill (milliseconds)
 */
void a_oscill_set_t_update_screen(struct a_oscill *oscill, short t_screen_update);

/**
 * a_oscill_get_t_update_screen - output value the field t_update_screen from struct oscill on stdout
 * @data: user data
 */
void a_oscill_get_t_update_screen(struct a_oscill *oscill, void *data);

/**
 * a_oscill_free - destroy struct a_oscill.
 */
void a_oscill_free(struct a_oscill **a_oscill);

/**
 * a_oscill_linear_approx - linear approximation input data
 * @newsample: new data for drawing in screen in struct a_oscill
 * NOTE: this function start up SDL timer (SDL_GetTicks(void)) and
 *       output screen from struct a_oscill on display computer
 */
void a_oscill_linear_approx(struct a_oscill *oscill, double newsample);


/**
 * a_oscill_new_circle_buff - create struct circle_buff
 * @buff_size: size of array in struct circle_buff (type data in array: short)
 *
 * return: NULL if error
 *         pointer to struct circle_buff else
 */
struct circle_buff *a_oscill_new_circle_buff(int buff_size);

/**
 * a_oscill_write_c_buff - write in circle_buff
 * @sample: writing data
 * NOTE: if error then write message in stderrr
 */
void a_oscill_write_c_buff(struct circle_buff *c_buff, short sample);

/**
 * a_oscill_read_c_buff- read in circle_buff
 * @c_buff: pointer to struct circle_buff
 *
 * return: A_OSCILL_ERR if error or not data in c_buff
 *         data from c_buff else
 * NOTE: if error then write message in stderrr
 */
short a_oscill_read_c_buff(struct circle_buff *c_buff);

/**
 * a_oscil_free_c_buff - destroy circle_buff
 */
void a_oscil_free_c_buff(struct circle_buff **c_buff);

/*
 * a_oscill_convert_signal - return integer number equal volt / volt_div
 */
short a_oscill_convert_signal(double volt, double volt_div);

/*
 * a_oscill_draw_screen - write in field bufp in screen data from oscill display
 * screencolor: color oscill display (red, blue, green, yellow)
 * for example: a_oscill_draw_screen(screen, red);
 */
void a_oscill_draw_screen(SDL_Surface *screen, int screencolor);


/*function for interaction with struct a_oscill */

/**
 * a_oscill_get_time_cmd - write in stdout value of field time_div in struct a_oscill
 * recommended: data = NULL
 */
void a_oscill_get_time_cmd(struct a_oscill *oscill, const char *data);

/**
 * a_oscill_set_time_cmd - read number from data and write in field time_div of struct a_oscill
 * @data: string where write number
 */
void a_oscill_set_time_cmd(struct a_oscill *oscill, const char *data);

/**
 * a_oscill_get_voltage_cmd - write in stdout value of field volt_div in struct a_oscill
 * recommended: data = NULL
 */
void a_oscill_get_voltage_cmd(struct a_oscill *oscill, const char *data);
/**
 * a_oscill_set_voltage_cmd - read number from data and write in field volt_div of struct a_oscill
 * @data: string where write number
 */
void a_oscill_set_voltage_cmd(struct a_oscill *oscill, const char *data);


/**
 * a_oscill_get_ccolor_cmd - write in stdout value of field ccolor in struct a_oscill
 * recommended: data = NULL
 */
void a_oscill_get_ccolor_cmd(struct a_oscill *oscill, const char *data);
/**
 * a_oscill_set_ccolor_cmd - read color name from data and write in field ccolor of struct a_oscill
 * @data: string where write name color(red, green, blue, yellow)
 */
void a_oscill_set_ccolor_cmd(struct a_oscill *oscill, const char *data);


/**
 * a_oscill_get_scolor_cmd - write in stdout value of field scolor in struct a_oscill
 * recommended: data = NULL
 */
void a_oscill_get_scolor_cmd(struct a_oscill *oscill, const char *data);
/**
 * a_oscill_set_scolor_cmd - read color name from data and write in field scolor of struct a_oscill
 * @data: string where write name color(red, green, blue, yellow)
 */
void a_oscill_set_scolor_cmd(struct a_oscill *oscill, const char *data);

/**
 * a_oscill_get_i_time_cmd - write in stdout value of field i_time_div in struct a_oscill
 * recommended: data = NULL
 */
void a_oscill_get_i_time_cmd(struct a_oscill *oscill, const char *data);
/**
 * a_oscill_set_i_time_cmd - read number from data and write in field i_time_div of struct a_oscill
 * @data: string where write number
 */
void a_oscill_set_i_time_cmd(struct a_oscill *oscill, const char *data);

/**
 * a_oscill_get_t_update_screen_cmd - write in stdout value of field t_screen_update in struct a_oscill
 * recommended: data = NULL
 */
void a_oscill_get_t_update_screen_cmd(struct a_oscill *oscill, const char *data);

/**
 * a_oscill_set_t_update_screen_cmd - read number from data and write in field t_screen_update of struct a_oscill
 * @data: string where write number
 */
void a_oscill_set_t_update_screen_cmd(struct a_oscill *oscill, const char *data);

/**
 * a_oscill_feed_samples_cmd - read samples in file or pipe
 * @data: name file (example: sin.txt) or pipe (example |./sin)
 * NOTE: if you wish read in pipe then first symbol in data must be '|'
 */
void a_oscill_feed_samples_cmd(struct a_oscill *oscill, const char *data);

/*
 * a_oscill_fread - read symbols from file and write them in buff
 * buff    : symbols array
 * sizebuff: size of buff
 *
 * return : number of read symbols if not error
 * 			A_OSCILL_ERROR if error
 */
int a_oscill_fread(FILE *fp, char *buff, int sizebuff);

#endif
